.Draftail-AI-LoadingOverlay {
  background-color: rgba(255, 255, 255, 0.5);
  -webkit-backdrop-filter: blur(4px);
          backdrop-filter: blur(4px);
  position: absolute;
  width: 100%;
  height: 100%;
  top: 0;
  left: 0;
  z-index: 100;
  display: flex;
  align-items: center;
  justify-content: center;
}

.Draftail-AI-LoadingOverlay > span > svg {
  margin-right: 5px;
}

.Draftail-AI-ButtonDropdown {
  position: absolute;
  border: 1px solid var(--w-color-grey-200);
  border-radius: 0.3125rem;
  background-color: white;
  display: flex;
  flex-direction: column;
  min-width: 300px;
  z-index: 200;
}

.Draftail-AI-ButtonDropdown > button {
  background-color: transparent;
  padding: 8px;
  text-align: left;
}

.Draftail-AI-ButtonDropdown > button span {
  font-weight: bold;
  display: block;
}

.Draftail-AI-ButtonDropdown > button:hover {
  background-color: var(--w-color-grey-200);
}

